@@ -1,18 +1,21 @@
-__version__ = '0.6.2'
+__version__ = '0.7.0'
 
 from yarl import URL
-from distutils.version import StrictVersion
+from distutils.version import StrictVersion  # type: ignore
 from typing import Union
 
+import async_timeout
 import asyncio
 import aiohttp
 import ipaddress
 import re
 import socket
 
-from multiaddr import Multiaddr
-from multiaddr.exceptions import ParseError
-from multiaddr.exceptions import StringParseError
+from aiohttp import BasicAuth
+
+from multiaddr import Multiaddr  # type: ignore
+from multiaddr.exceptions import ParseError  # type: ignore
+from multiaddr.exceptions import StringParseError  # type: ignore
 
 from aioipfs import api
 from aioipfs.exceptions import *  # noqa
@@ -32,6 +35,18 @@ class IpfsDaemonVersion(StrictVersion):
     pass
 
 
+class BearerAuth(BasicAuth):
+    """
+    Cheap subclass of BasicAuth to authenticate with a token
+    """
+
+    def __init__(self, token: str):
+        self.token = token
+
+    def encode(self) -> str:
+        return f'Bearer {self.token}'
+
+
 class AsyncIPFS(object):
     """
     Asynchronous IPFS API client
@@ -56,6 +71,7 @@ class AsyncIPFS(object):
                  port: int = RPC_API_DEFAULT_PORT,
                  scheme: str = 'http',
                  maddr: Union[Multiaddr, str] = None,
+                 auth: Union[BasicAuth, BearerAuth, None] = None,
                  loop=None,
                  conns_max: int = 0,
                  conns_max_per_host: int = 0,
@@ -75,6 +91,8 @@ class AsyncIPFS(object):
                                                     scheme=scheme,
                                                     maddr=maddr,
                                                     api_version=api_version)
+        self.__auth = auth if isinstance(auth,
+                                         (BasicAuth, BearerAuth)) else None
 
         self.session = self.get_session()
 
@@ -196,6 +214,17 @@ class AsyncIPFS(object):
     def commands(self):
         return self.core.commands
 
+    @property
+    def auth(self):
+        return self.__auth
+
+    @auth.setter
+    def auth(self, authh: Union[BasicAuth, BearerAuth, None]):
+        if authh and not isinstance(authh, (BasicAuth, BearerAuth)):
+            raise ValueError('Invalid authentication helper')
+
+        self.__auth = authh
+
     def __compute_base_api_url(self, host=None, port=None, scheme='http',
                                maddr=None, api_version='v0'):
         """
@@ -324,6 +353,12 @@ class AsyncIPFS(object):
             # Shouldn't assume dns4 here
             return Multiaddr(f'/dns4/{self.host}/tcp/{port}')
 
+    def timeout(self, time: float) -> async_timeout.Timeout:
+        return async_timeout.timeout(time)
+
+    def timeout_at(self, time: float) -> async_timeout.Timeout:
+        return async_timeout.timeout_at(time)
+
     async def __aenter__(self):
         return self
 
