@@ -2,14 +2,16 @@ import os.path
 import tarfile
 import tempfile
 import sys
+from typing import Union, Optional, Dict
 from pathlib import Path
 
 import asyncio
-import aiofiles
+import aiofiles  # type: ignore
 import aioipfs
 
 from aiohttp import payload
 
+from .apis import HTTP_ERROR_CODES
 from .apis import SubAPI
 from . import multi
 from .helpers import *  # noqa
@@ -248,7 +250,7 @@ class ConfigAPI(SubAPI):
         :param bool json: Parse stringified JSON
         """
 
-        params = {}
+        params: Dict = {}
 
         if value is not None:
             if boolean:
@@ -256,7 +258,7 @@ class ConfigAPI(SubAPI):
             else:
                 params[ARG_PARAM] = [key, value]
         else:
-            params = {ARG_PARAM: key}
+            params[ARG_PARAM] = key
 
         params['bool'] = boolarg(boolean)
         params['json'] = boolarg(json)
@@ -359,6 +361,9 @@ class DhtAPI(SubAPI):
             yield value
 
     async def query(self, peerid, verbose=False):
+        """
+        DEPRECATED: This command is deprecated
+        """
         async for value in self.mjson_decode(
                 self.url('dht/query'),
                 params={ARG_PARAM: peerid, 'verbose': boolarg(verbose)}):
@@ -387,18 +392,18 @@ class DiagAPI(SubAPI):
         return await self.fetch_text(self.url('diag/cmds/clear'))
 
     async def profile(self,
-                      output: str = None,
-                      collectors: list = None,
-                      profile_time: str = None,
-                      mutex_profile_fraction: int = None,
-                      block_profile_rate: str = None):
+                      output: Optional[str] = None,
+                      collectors: Optional[list] = None,
+                      profile_time: Optional[str] = None,
+                      mutex_profile_fraction: Optional[int] = None,
+                      block_profile_rate: Optional[str] = None):
         """
         Collect a performance profile for debugging.
 
         TODO: support passing collectors as an array
         """
 
-        params = {}
+        params: Dict = {}
 
         if isinstance(output, str):
             params['output'] = output
@@ -439,7 +444,7 @@ class FilesAPI(SubAPI):
 
     async def chcid(self, path: str,
                     cidversion: int,
-                    hashfn: str = None):
+                    hashfn: Optional[str] = None):
         params = {ARG_PARAM: path, 'cid-version': str(cidversion)}
 
         if isinstance(hashfn, str):
@@ -452,11 +457,11 @@ class FilesAPI(SubAPI):
         Flush a given path's data to disk.
         """
         params = {ARG_PARAM: path}
-        return await self.fetch_text(self.url('files/flush'),
+        return await self.fetch_json(self.url('files/flush'),
                                      params=params)
 
     async def mkdir(self, path, parents=False, cid_version=None,
-                    hashfn: str = None):
+                    hashfn: Optional[str] = None):
         params = {ARG_PARAM: path, 'parents': boolarg(parents)}
 
         if isinstance(hashfn, str):
@@ -483,8 +488,8 @@ class FilesAPI(SubAPI):
                                      params=params)
 
     async def read(self, path,
-                   offset: int = None,
-                   count: int = None):
+                   offset: Optional[int] = None,
+                   count: Optional[int] = None):
         params = {ARG_PARAM: path}
 
         if offset is not None and isinstance(offset, int):
@@ -523,7 +528,7 @@ class FilesAPI(SubAPI):
     async def write(self, mfspath, data, create=False,
                     parents=False, cid_version=None,
                     raw_leaves=False,
-                    hashfn: str = None,
+                    hashfn: Optional[str] = None,
                     truncate=False, offset=-1, count=-1):
         """
         Write to a mutable file in a given filesystem.
@@ -602,7 +607,7 @@ class KeyAPI(SubAPI):
     async def key_import(self,
                          keypath: str,
                          name,
-                         ipns_base: str = None,
+                         ipns_base: Optional[str] = None,
                          format='libp2p-protobuf-cleartext',
                          allow_any_keytype: bool = False):
         """
@@ -656,7 +661,7 @@ class KeyAPI(SubAPI):
                                      params=params)
 
     async def list(self, long=False,
-                   ipns_base: str = None):
+                   ipns_base: Optional[str] = None):
         params = {'l': boolarg(long)}
 
         if isinstance(ipns_base, str):
@@ -665,8 +670,8 @@ class KeyAPI(SubAPI):
         return await self.fetch_json(self.url('key/list'),
                                      params=params)
 
-    async def gen(self, name, type='rsa', size: int = 2048,
-                  ipns_base: str = None):
+    async def gen(self, name, type='ed25519', size: int = 2048,
+                  ipns_base: Optional[str] = None):
         params = {ARG_PARAM: name, 'type': type, 'size': str(size)}
 
         if isinstance(ipns_base, str):
@@ -676,7 +681,7 @@ class KeyAPI(SubAPI):
                                      params=params)
 
     async def rm(self, name,
-                 ipns_base: str = None):
+                 ipns_base: Optional[str] = None):
         params = {ARG_PARAM: name}
 
         if isinstance(ipns_base, str):
@@ -684,7 +689,8 @@ class KeyAPI(SubAPI):
 
         return await self.fetch_json(self.url('key/rm'), params=params)
 
-    async def rename(self, src, dst, ipns_base=None, force: bool = False):
+    async def rename(self, src, dst, ipns_base: Optional[str] = None,
+                     force: bool = False):
         params = {
             ARG_PARAM: [src, dst],
             'force': boolarg(force)
@@ -757,10 +763,43 @@ class NameAPI(SubAPI):
 
         self.pubsub = NamePubsubAPI(driver)
 
+    async def inspect(self,
+                      record: Union[str, Path],
+                      verify: Optional[str] = None,
+                      dump: bool = True):
+        """
+        Inspects an IPNS Record
+
+        :param record: Path to the file containing the IPNS record
+        :type record: Union[Path, str]
+        :param str verify: CID of the public IPNS key to validate against
+        :param bool dump: Include a full hex dump of the raw Protobuf record
+        """
+
+        if not isinstance(record, str) and not isinstance(record, Path):
+            raise ValueError('Invalid value type: should be Path or str')
+
+        if (isinstance(record, str) and not os.path.isfile(record)) or \
+           (isinstance(record, Path) and not record.is_file()):
+            raise ValueError(f'Invalid IPNS record file path: {record}')
+
+        params = {
+            'dump': boolarg(dump)
+        }
+
+        if isinstance(verify, str):
+            params['verify'] = verify
+
+        with multi.FormDataWriter() as mpwriter:
+            mpwriter.append_payload(multi.bytes_payload_from_file(str(record)))
+
+            return await self.post(self.url('name/inspect'),
+                                   mpwriter, params=params, outformat='json')
+
     async def publish(self, path, resolve=True, lifetime='24h',
                       key='self', ttl=None,
                       quieter=False, allow_offline=False,
-                      ipns_base: str = None):
+                      ipns_base: Optional[str] = None):
         """
         Publish IPNS names
 
@@ -985,11 +1024,11 @@ class ObjectAPI(SubAPI):
 class RefsAPI(SubAPI):
     async def refs(self,
                    path: str,
-                   format: str = None,
+                   format: Optional[str] = None,
                    edges: bool = False,
                    unique: bool = False,
                    recursive: bool = False,
-                   max_depth: int = None):
+                   max_depth: Optional[int] = None):
         """
         List links (references) from an object.
         """
@@ -1209,6 +1248,7 @@ class CoreAPI(SubAPI):
 
     def _add_post(self, data, params=None):
         return self.driver.session.post(self.url('add'), data=data,
+                                        auth=self.driver.auth,
                                         params=params if params else {})
 
     async def add_single(self, mpart, params=None):
@@ -1218,6 +1258,9 @@ class CoreAPI(SubAPI):
 
         async with self._add_post(
                 mpart, params=params if params else {}) as response:
+            if response.status in HTTP_ERROR_CODES:
+                self.handle_error(response, await response.text())
+
             return await response.json()
 
     async def add_generic(self, mpart, params=None):
@@ -1390,9 +1433,9 @@ class CoreAPI(SubAPI):
             params={'flags': boolarg(flags)}
         )
 
-    async def id(self, peer: str = None,
-                 format: str = None,
-                 peerid_base: str = None):
+    async def id(self, peer: Optional[str] = None,
+                 format: Optional[str] = None,
+                 peerid_base: Optional[str] = None):
         """
         Show IPFS node id info.
 
@@ -1426,32 +1469,43 @@ class CoreAPI(SubAPI):
 
         return await self.fetch_raw(self.url('cat'), params=params)
 
-    def _extract_tar(self, archive_path: str, dstdir: str, compress: bool):
-        # Synchronous tar extraction runs in the executor
+    def __remove_tar(self, archive_fd: int, archive_path: str) -> None:
+        try:
+            os.close(archive_fd)
+            os.unlink(archive_path)
+        except Exception as err:
+            print(
+                f'Could not remove TAR file: {archive_path}: {err}',
+                file=sys.stderr
+            )
+
+    def _extract_tar(self,
+                     archive_path: str,
+                     dstdir: str,
+                     compress: bool) -> bool:
+        """
+        Synchronous tar extraction runs in the threadpool executor
+        """
 
         mode = 'r|gz' if compress is True else 'r|'
 
-        def remove_archive():
-            try:
-                os.unlink(archive_path)
-            except Exception as err:
-                print('Could not remove TAR file:', str(err), file=sys.stderr)
-
         try:
             with tarfile.open(name=archive_path, mode=mode) as tf:
                 tf.extractall(path=dstdir)
         except Exception as err:
-            print('Could not extract TAR file:', str(err), file=sys.stderr)
-            remove_archive()
+            print(
+                f'Failed to extract TAR file: {archive_path}: {err}',
+                file=sys.stderr
+            )
+
             return False
         else:
-            remove_archive()
             return True
 
     async def get(self, path, dstdir='.', compress=False,
                   compression_level=-1, archive=True, output=None,
                   progress_callback=None, progress_callback_arg=None,
-                  chunk_size=16384):
+                  chunk_size=262144):
         """
         Download IPFS objects.
 
@@ -1460,6 +1514,7 @@ class CoreAPI(SubAPI):
         :param bool compress: Compress the output with GZIP compression
         :param str compression_level: The level of compression (1-9)
         :param bool archive: Output a TAR archive
+        :param int chunk_size: HTTP chunk size
         """
 
         params = {
@@ -1472,7 +1527,7 @@ class CoreAPI(SubAPI):
         if isinstance(output, str):
             params['output'] = output
 
-        archive_path = tempfile.mkstemp(prefix='aioipfs')[1]
+        archive_fd, archive_path = tempfile.mkstemp(prefix='aioipfs')
 
         # We read chunk by chunk the tar data coming from the
         # daemon and use aiofiles to asynchronously write the data to
@@ -1503,15 +1558,22 @@ class CoreAPI(SubAPI):
 
         # Run the tar extraction inside asyncio's threadpool
         loop = asyncio.get_event_loop()
-        tar_future = loop.run_in_executor(
+
+        result = await loop.run_in_executor(
             None, self._extract_tar,
-            archive_path, dstdir, compress)
-        return await tar_future
+            archive_path,
+            dstdir,
+            compress
+        )
+
+        self.__remove_tar(archive_fd, archive_path)
+
+        return result
 
     async def getgen(self, objpath, dstdir='.', compress=False,
                      compression_level=-1, archive=True, output=None,
                      progress_callback=None, progress_callback_arg=None,
-                     sleept=0.05, chunk_size=16384):
+                     sleept=0.05, chunk_size=262144):
         """
         Download IPFS objects (async generator)
 
@@ -1520,6 +1582,7 @@ class CoreAPI(SubAPI):
         :param bool compress: Compress the output with GZIP compression
         :param str compression_level: The level of compression (1-9)
         :param bool archive: Output a TAR archive
+        :param int chunk_size: HTTP chunk size
         """
 
         params = {
@@ -1532,7 +1595,7 @@ class CoreAPI(SubAPI):
         if isinstance(output, str):
             params['output'] = output
 
-        archive_path = tempfile.mkstemp(prefix='aioipfs')[1]
+        archive_fd, archive_path = tempfile.mkstemp(prefix='aioipfs')
 
         # We read chunk by chunk the tar data coming from the
         # daemon and use aiofiles to asynchronously write the data to
@@ -1576,9 +1639,14 @@ class CoreAPI(SubAPI):
             loop = asyncio.get_event_loop()
             tar_future = loop.run_in_executor(
                 None, self._extract_tar,
-                archive_path, dstdir, compress)
+                archive_path,
+                dstdir,
+                compress
+            )
             res = await tar_future
 
+            self.__remove_tar(archive_fd, archive_path)
+
             if res is True:
                 yield 1, None, None
             else:
@@ -1683,8 +1751,8 @@ class CoreAPI(SubAPI):
 
     async def resolve(self, name,
                       recursive: bool = False,
-                      dht_record_count: int = None,
-                      dht_timeout: int = None):
+                      dht_record_count: Optional[int] = None,
+                      dht_timeout: Optional[int] = None):
         """
         Resolve the value of names to IPFS
 
