@@ -2,6 +2,8 @@ import json
 import asyncio
 import sys
 
+from typing import Union
+
 from aiohttp.web_exceptions import (HTTPInternalServerError,
                                     HTTPForbidden,
                                     HTTPNotFound,
@@ -11,6 +13,7 @@ from aiohttp.client_exceptions import *  # noqa
 
 from aioipfs.helpers import *  # noqa
 from aioipfs.exceptions import *  # noqa
+from aioipfs.util import car_decoder, have_car_decoder
 
 
 DEFAULT_TIMEOUT = 60 * 60
@@ -50,7 +53,7 @@ class SubAPI:
         except Exception:
             return None, None
 
-    def handle_error(self, response, data):
+    def handle_error(self, response, data: Union[str, None]):
         """
         When the daemon returns an HTTP status code != 200, this
         method is called to raise an API exception.
@@ -64,6 +67,10 @@ class SubAPI:
 
         msg, code = self.decode_error(data)
 
+        if response.status == HTTPForbidden.status_code:
+            # 403
+            raise RPCAccessDenied(str(msg))
+
         if isinstance(msg, str) and code is not None:
             # Check for specific errors
 
@@ -87,7 +94,8 @@ class SubAPI:
             raise UnknownAPIError()
 
     async def fetch_text(self, url, params={}, timeout=DEFAULT_TIMEOUT):
-        async with self.driver.session.post(url, params=params) as response:
+        async with self.driver.session.post(url, params=params,
+                                            auth=self.driver.auth) as response:
             status, textdata = response.status, await response.text()
             if status in HTTP_ERROR_CODES:
                 self.handle_error(response, textdata)
@@ -95,13 +103,29 @@ class SubAPI:
             return textdata
 
     async def fetch_raw(self, url, params={}, timeout=DEFAULT_TIMEOUT):
-        async with self.driver.session.post(url, params=params) as response:
+        async with self.driver.session.post(url, params=params,
+                                            auth=self.driver.auth) as response:
             status, data = response.status, await response.read()
             if status in HTTP_ERROR_CODES:
                 self.handle_error(response, data)
 
             return data
 
+    async def car_stream(self, url, params={}):
+        if not have_car_decoder:
+            raise Exception('The CAR decoding library is not available')
+
+        stream = car_decoder.ChunkedMemoryByteStream()
+
+        async with self.driver.session.post(url, params=params,
+                                            auth=self.driver.auth) as resp:
+            async for chunk, _ in resp.content.iter_chunks():
+                await stream.append_bytes(chunk)
+
+        await stream.mark_complete()
+
+        return stream
+
     async def fetch_json(self, url, params={}, timeout=DEFAULT_TIMEOUT):
         return await self.post(url, params=params, outformat='json')
 
@@ -109,6 +133,7 @@ class SubAPI:
                    outformat='text'):
         try:
             async with self.driver.session.post(url, data=data,
+                                                auth=self.driver.auth,
                                                 headers=headers,
                                                 params=params) as response:
                 if response.status in HTTP_ERROR_CODES:
@@ -133,8 +158,7 @@ class SubAPI:
             raise apierr
         except (ClientPayloadError,
                 ClientConnectorError,
-                ServerDisconnectedError,
-                BaseException) as err:
+                ServerDisconnectedError) as err:
             if self.driver.debug:
                 print(f'{url}: aiohttp error: {err}',
                       file=sys.stderr)
@@ -155,7 +179,7 @@ class SubAPI:
         :param params: http params
         """
 
-        kwargs = {'params': params if params else {}}
+        kwargs = {'params': params if params else {}, 'auth': self.driver.auth}
 
         if new_session is True:
             session = self.driver.get_session(
@@ -177,6 +201,9 @@ class SubAPI:
 
         try:
             async with getattr(session, method)(url, **kwargs) as response:
+                if response.status in HTTP_ERROR_CODES:
+                    self.handle_error(response, await response.text())
+
                 async for raw_message in response.content:
                     message = decode_json(raw_message)
 
@@ -200,8 +227,7 @@ class SubAPI:
             raise e
         except (ClientPayloadError,
                 ClientConnectorError,
-                ServerDisconnectedError,
-                Exception):
+                ServerDisconnectedError):
             if new_session is True:
                 await session.close()
 
