@@ -1,9 +1,10 @@
 import os
 import os.path
+import posixpath
 import io
 import re
 
-from aioipfs.gitignore_parser import parse_gitignore
+from gitignore_parser import parse_gitignore  # type: ignore
 import aiohttp
 from aiohttp import payload
 
@@ -46,7 +47,7 @@ def bytes_payload_from_file(filepath):
     return file_payload
 
 
-def glob_compile(pat):
+def glob_compile(pat):  # pragma: no cover
     """ From ipfsapi.multipart
 
        Translate a shell glob PATTERN to a regular expression.
@@ -95,7 +96,7 @@ def glob_compile(pat):
                 res = '%s[%s]' % (res, stuff)
         else:
             res = res + re.escape(c)
-    return re.compile('^' + res + r'\Z(?ms)' + '$')
+    return re.compile(r'(?ms)^' + res + r'\Z' + '$')
 
 
 class DirectoryListing:
@@ -159,7 +160,9 @@ class DirectoryListing:
                 return
 
             dir_unr = strip_first(short_path)
-            if dir_unr and self.ignrules and self.ignrules(dir_unr):
+            dir_fpath = os.path.join(self.directory, dir_unr)
+
+            if dir_unr and self.ignrules and self.ignrules(dir_fpath):
                 # Matches ignore rules
                 return
 
@@ -183,6 +186,10 @@ class DirectoryListing:
                 # Create an empty, fake file to represent the directory
                 mock_file = io.StringIO()
                 mock_file.write(u'')
+
+                # posix-ify
+                dir_base = dir_base.replace(os.sep, posixpath.sep)
+
                 # Add this directory to those that will be sent
                 names.append(('files',
                               (dir_base,
@@ -199,10 +206,13 @@ class DirectoryListing:
                     return
 
                 short_unr = strip_first(short_path)
-                if short_unr and self.ignrules and self.ignrules(short_unr):
+                if short_unr and self.ignrules and self.ignrules(full_path):
                     # Matches ignore rules
                     return
 
+                # posix-ify
+                short_path = short_path.replace(os.sep, posixpath.sep)
+
                 # Always add files in wildcard directories
                 names.append(('files', (short_path,
                                         open(full_path, 'rb'),
@@ -239,7 +249,11 @@ class DirectoryListing:
         if self.ign_path:
             if os.path.exists(self.ign_path) and not self.ignrules:
                 try:
-                    self.ignrules = parse_gitignore(self.ign_path)
+                    self.ignrules = parse_gitignore(
+                        self.ign_path,
+                        self.directory
+                    )
+
                 except Exception:
                     self.ignrules = None
 
