@@ -1,13 +1,14 @@
+import base64
 import pytest
 
 import tempfile
 import random
 import string
-import time
-import subprocess
 import os
 import os.path
+import platform
 import json
+import sys
 
 from pathlib import Path
 from multiaddr import Multiaddr
@@ -15,43 +16,9 @@ from multiaddr import Multiaddr
 import asyncio
 import aioipfs
 
-
-def ipfs_getconfig_var(var):
-    sp_getconfig = subprocess.Popen(['ipfs', 'config',
-                                     var], stdout=subprocess.PIPE)
-    stdout, stderr = sp_getconfig.communicate()
-    return stdout.decode()
-
-
-@pytest.fixture
-def datafiles():
-    return Path(os.path.dirname(__file__)).joinpath('datafiles')
-
-
-@pytest.fixture
-def smalltar():
-    import tarfile
-    tpath = tempfile.mkstemp()[1]
-    tar = tarfile.open(tpath, 'w|')
-    tar.add(__file__)
-    tar.close()
-    yield tar, tpath
-    os.unlink(tpath)
-
-
-@pytest.fixture
-def testfile1(tmpdir):
-    filep = tmpdir.join('testfile1.txt')
-    filep.write('POIEKJDOOOPIDMWOPIMPOWE()=ds129084bjcy')
-    return filep
-
-
-@pytest.fixture
-def testfile2(tmpdir):
-    r = random.Random()
-    filep = tmpdir.join('testfile2.txt')
-    filep.write(''.join([str(r.randint(i, i * 2)) for i in range(0, 16)]))
-    return filep
+from aioipfs import util
+from aioipfs.multi import DirectoryListing
+from aioipfs.exceptions import RPCAccessDenied
 
 
 def random_word(length=8):
@@ -59,98 +26,6 @@ def random_word(length=8):
         random.choice(string.ascii_lowercase) for c in range(length))
 
 
-@pytest.fixture
-def hiddendirs(tmpdir):
-    root = Path(str(tmpdir))
-    root.joinpath('a/b/.c').mkdir(parents=True)
-    root.joinpath('a/b/.c/file0').touch()
-    root.joinpath('d/.e/f').mkdir(parents=True)
-    root.joinpath('d/.e/f/.file3').touch()
-    root.joinpath('file1').touch()
-    root.joinpath('.file2').touch()
-    return str(root)
-
-
-@pytest.fixture
-def ignoredirs(tmpdir):
-    root = Path(str(tmpdir))
-    root.joinpath('a/b/.c').mkdir(parents=True)
-    root.joinpath('d/.e/f').mkdir(parents=True)
-    root.joinpath('d/.e/f/.file3').touch()
-    root.joinpath('file1').touch()
-    root.joinpath('README.txt').touch()
-    root.joinpath('README2.txt').touch()
-    root.joinpath('.file2').touch()
-    ign = root.joinpath('.gitignore')
-    ign.touch()
-    ign.write_text("README.txt\n.file2\na**\n\nd/.e/*/*\nd/.e/f\n")
-    return str(root)
-
-
-def ipfs_config(param, value):
-    os.system('ipfs config {0} "{1}"'.format(param, value))
-
-
-def ipfs_config_json(param, value):
-    os.system("ipfs config --json '{0}' '{1}'".format(
-        param, json.dumps(value)))
-
-
-apiport = 9001
-gwport = 9080
-swarmport = 9002
-
-
-@pytest.fixture(scope='module')
-def ipfsdaemon():
-    # Starts a daemon on high port and temporary directory, yield it
-    # when started and shut it down on fixture's exit
-
-    tmpdir = tempfile.mkdtemp()
-
-    # Setup IPFS_PATH and initialize the repository
-    os.putenv('IPFS_PATH', tmpdir)
-    os.system('ipfs init -e')
-
-    ipfs_config_json(
-        'Addresses.API', [
-            f'/ip4/127.0.0.1/tcp/{apiport}',
-            f'/ip6/::1/tcp/{apiport}',
-        ]
-    )
-
-    ipfs_config('Addresses.Gateway',
-                '/ip4/127.0.0.1/tcp/{0}'.format(gwport))
-    ipfs_config_json('Addresses.Swarm',
-                     [f"/ip4/127.0.0.1/tcp/{swarmport}"])
-
-    # Empty bootstrap so we're not bothered
-    ipfs_config_json('Bootstrap', [])
-    ipfs_config_json('Experimental.Libp2pStreamMounting', True)
-    ipfs_config_json('Experimental.FilestoreEnabled', True)
-
-    # Run the daemon and wait a bit
-    sp = subprocess.Popen(['ipfs', 'daemon', '--enable-pubsub-experiment'],
-                          stdout=subprocess.PIPE)
-    time.sleep(1)
-
-    yield tmpdir, sp
-
-    time.sleep(0.5)
-    # Cleanup
-    sp.terminate()
-
-
-@pytest.fixture
-def ipfs_peerid(ipfsdaemon):
-    return ipfs_getconfig_var('Identity.PeerID').strip()
-
-
-@pytest.fixture()
-def iclient(event_loop):
-    return aioipfs.AsyncIPFS(port=apiport, loop=event_loop)
-
-
 class TestClientConstructor:
     @pytest.mark.asyncio
     async def test_invalid_constructor(self, event_loop):
@@ -181,8 +56,27 @@ class TestClientConstructor:
         with pytest.raises(aioipfs.InvalidNodeAddressError):
             aioipfs.AsyncIPFS(maddr='/ip4/127.0.0.1/udp/4000', loop=event_loop)
 
+        # Invalid application layer protocol
+        with pytest.raises(aioipfs.InvalidNodeAddressError):
+            aioipfs.AsyncIPFS(maddr='/ip4/localhost/tcp/4000/invalid',
+                              loop=event_loop)
+
     @pytest.mark.asyncio
     async def test_constructor_apiurl(self, event_loop, ipfsdaemon):
+        ddir, apiport, sp = ipfsdaemon
+
+        # Test by passing a host and port
+        client = aioipfs.AsyncIPFS(
+            host='localhost', port=apiport, loop=event_loop)
+
+        assert str(client.api_url) == f'http://localhost:{apiport}/api/v0/'
+
+        # Test by passing a host, port and scheme
+        client = aioipfs.AsyncIPFS(
+            host='localhost', port=apiport, scheme='https', loop=event_loop)
+
+        assert str(client.api_url) == f'https://localhost:{apiport}/api/v0/'
+
         # Test by passing a valid /ip4/x.x.x.x/tcp/port multiaddr
         client = aioipfs.AsyncIPFS(
             maddr=f'/ip4/127.0.0.1/tcp/{apiport}', loop=event_loop)
@@ -201,6 +95,12 @@ class TestClientConstructor:
         )
         assert str(client.api_url) == f'http://example.com:{apiport}/api/v0/'
 
+        # Test by passing a valid HTTPS multiaddr
+        client = aioipfs.AsyncIPFS(
+            maddr=f'/dns4/localhost/tcp/{apiport}/https', loop=event_loop)
+
+        assert str(client.api_url) == f'https://localhost:{apiport}/api/v0/'
+
         # Test by passing a valid /ip6/.../tcp/port multiaddr
         client = aioipfs.AsyncIPFS(
             maddr=f'/ip6/::1/tcp/{apiport}', loop=event_loop)
@@ -222,21 +122,43 @@ class TestClientConstructor:
         client = aioipfs.AsyncIPFS()
         assert str(client.api_url) == 'http://localhost:5001/api/v0/'
 
+    @pytest.mark.asyncio
+    async def test_constructor_auth(self, event_loop):
+        client = aioipfs.AsyncIPFS(auth=aioipfs.BasicAuth('bob', 'pwd'))
+        assert client.auth.login == 'bob'
+        assert client.auth.password == 'pwd'
+
+        clif = aioipfs.AsyncIPFS()
+        with pytest.raises(ValueError):
+            clif.auth = 'basic:test:test'
+
+        clif.auth = aioipfs.BearerAuth('secret-token')
+        assert clif.auth.token == 'secret-token'
+
+        clif.auth = None
+        assert clif.auth is None
+
 
 class TestAsyncIPFS:
     @pytest.mark.asyncio
     async def test_basic(self, event_loop, ipfsdaemon, iclient):
-        tmpdir, sp = ipfsdaemon
         await iclient.id()
         await iclient.core.version()
         await iclient.commands()
-        await iclient.close()
+
+    @pytest.mark.asyncio
+    async def test_timeout(self, event_loop, iclient):
+        with pytest.raises(asyncio.TimeoutError):
+            async with iclient.timeout(1):
+                await asyncio.sleep(3)
+
+        with pytest.raises(asyncio.TimeoutError):
+            async with iclient.timeout_at(event_loop.time() + 2):
+                await asyncio.sleep(4)
 
     @pytest.mark.asyncio
     async def test_bootstrap(self, event_loop, ipfsdaemon, iclient):
-        tmpdir, sp = ipfsdaemon
         await iclient.bootstrap.list()
-        await iclient.close()
 
     @pytest.mark.asyncio
     async def test_swarm(self, event_loop, ipfsdaemon, iclient):
@@ -244,7 +166,15 @@ class TestAsyncIPFS:
         await iclient.swarm.addrs()
         await iclient.swarm.addrs_local()
         await iclient.swarm.addrs_listen()
-        await iclient.close()
+
+    @pytest.mark.asyncio
+    async def test_swarm_resources(self, event_loop, ipfsdaemon, iclient):
+        if await iclient.agent_version_get() < \
+                aioipfs.IpfsDaemonVersion('0.20.0'):
+            # /api/v0/swarm/resources was introduced in kubo v0.19.0
+            pytest.skip('RPC endpoint not available')
+
+        assert 'System' in await iclient.swarm.resources()
 
     @pytest.mark.asyncio
     async def test_swarm_peering(self, event_loop, ipfsdaemon, iclient):
@@ -263,8 +193,6 @@ class TestAsyncIPFS:
         with pytest.raises(aioipfs.APIError):
             await iclient.swarm.peering.rm('nothere')
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     async def test_refs(self, event_loop, ipfsdaemon, iclient,
                         testfile1):
@@ -276,14 +204,11 @@ class TestAsyncIPFS:
         async for refobj in iclient.refs.local():
             assert 'Ref' in refobj
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     async def test_block1(self, event_loop, ipfsdaemon, iclient, testfile1):
         reply = await iclient.block.put(testfile1)
         data = await iclient.block.get(reply['Key'])
         assert data.decode() == testfile1.read()
-        await iclient.close()
 
     @pytest.mark.asyncio
     async def test_add(self, event_loop, ipfsdaemon, iclient, testfile1,
@@ -325,17 +250,49 @@ class TestAsyncIPFS:
             await iclient.add_str('test', to_files='/wslash')
             assert (await iclient.files.read('/wslash')).decode() == 'test'
 
-        await iclient.close()
+    @pytest.mark.asyncio
+    async def test_auth(self, event_loop, ipfsdaemon_with_auth,
+                        ipfs_version,
+                        iclient_with_auth, testfile1):
+        if ipfs_version < aioipfs.IpfsDaemonVersion('0.25.0'):
+            # RPC Authorization was introduced in kubo v0.25.0
+            pytest.skip('RPC Authorization not supported ')
+
+        iclient_with_auth.auth = aioipfs.BasicAuth('alice', 'password123')
+
+        # alice doesn't have access to the 'id' RPC API
+        with pytest.raises(RPCAccessDenied):
+            await iclient_with_auth.core.id()
+
+        # alice can use the 'files' API
+        assert await iclient_with_auth.files.ls('/')
+
+        # alice doesn't have access to the 'add' RPC API
+        with pytest.raises(RPCAccessDenied):
+            cids = [added['Hash'] async for added in iclient_with_auth.add(
+                str(testfile1))]
+
+        # john, however, does
+        iclient_with_auth.auth = aioipfs.BasicAuth('john', '12345')
+        cids = [added['Hash'] async for added in iclient_with_auth.add(
+            str(testfile1))]
+        assert len(cids) == 1
+
+        # The token has access to the whole APi
+        iclient_with_auth.auth = aioipfs.BearerAuth('token123')
+        assert await iclient_with_auth.files.ls('/')
+        assert await iclient_with_auth.core.id()
 
     @pytest.mark.asyncio
-    async def test_hidden(self, event_loop, ipfsdaemon, iclient, hiddendirs):
-        async for added in iclient.add(hiddendirs, hidden=False):
+    async def test_hidden(self, event_loop, ipfsdaemon, iclient,
+                          dir_hierarchy1):
+        async for added in iclient.add(dir_hierarchy1, hidden=False):
             parts = added['Name'].split('/')
             for part in parts:
                 assert not part.startswith('.')
 
         names = []
-        async for added in iclient.add(hiddendirs, hidden=True):
+        async for added in iclient.add(dir_hierarchy1, hidden=True):
             names.append(added['Name'])
 
         assert 'test_hidden0/d/.e/f/.file3' in names
@@ -343,9 +300,9 @@ class TestAsyncIPFS:
 
     @pytest.mark.asyncio
     async def test_ignorerules(self, event_loop, ipfsdaemon, iclient,
-                               ignoredirs):
+                               dir_hierarchy2):
         names = []
-        async for added in iclient.add(ignoredirs,
+        async for added in iclient.add(str(dir_hierarchy2),
                                        ignore_rules_path='.gitignore',
                                        hidden=True):
             names.append(added['Name'])
@@ -358,7 +315,7 @@ class TestAsyncIPFS:
         assert 'test_ignorerules0/README2.txt' in names
 
         names = []
-        async for added in iclient.add(ignoredirs,
+        async for added in iclient.add(str(dir_hierarchy2),
                                        ignore_rules_path='.gitignore',
                                        hidden=False):
             names.append(added['Name'])
@@ -368,13 +325,14 @@ class TestAsyncIPFS:
     @pytest.mark.asyncio
     async def test_addtar(self, event_loop, ipfsdaemon, iclient,
                           tmpdir, smalltar):
-        tar, tarpath = smalltar
-        reply = await iclient.tar.add(tarpath)
-        tarhash = reply['Hash']
-        fetched = await iclient.tar.cat(tarhash)
-        f = tmpdir.join('new.tar')
-        f.write(fetched)
-        await iclient.close()
+        if await iclient.agent_version_get() < \
+                aioipfs.IpfsDaemonVersion('0.26.0'):
+            tar, tarpath = smalltar
+            reply = await iclient.tar.add(tarpath)
+            tarhash = reply['Hash']
+            fetched = await iclient.tar.cat(tarhash)
+            f = tmpdir.join('new.tar')
+            f.write(fetched)
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize('order', ['gin', 'tonic'])
@@ -392,7 +350,6 @@ class TestAsyncIPFS:
 
         data = await iclient.cat(h)
         assert data.decode() == json.dumps(json1)
-        await iclient.close()
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize('data', [b'234098dsfkj2doidf0'])
@@ -407,8 +364,6 @@ class TestAsyncIPFS:
         reply = await iclient.add_bytes(data, cid_version=1, hash='sha2-512')
         assert reply['Hash'] == 'bafkrgqdao6vujlzh4z6o7mzgv3jnydftv2of5jy32yufswk7bnvwaq7oyaizo6gnditr4okfphi2cguz2cack27rsjfzuybm57knagzjl6m34'  # noqa
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     @pytest.mark.parametrize('data', [b'234098dsfkj2doidf0'])
     async def test_dag(self, event_loop, ipfsdaemon, iclient, tmpdir, data):
@@ -420,7 +375,6 @@ class TestAsyncIPFS:
 
         reply = await iclient.dag.put(filedag)
         assert 'Cid' in reply
-        await iclient.close()
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize('data', [b'234098dsfkj2doidf0'])
@@ -440,7 +394,7 @@ class TestAsyncIPFS:
         assert imported['Root']['Cid']['/'] is not None
         assert reply['Cid']['/'] == imported['Root']['Cid']['/']
 
-        filecar = tempfile.mkstemp()[1]
+        carfd, filecar = tempfile.mkstemp()
         with open(filecar, 'wb') as fd:
             fd.write(export)
 
@@ -448,13 +402,54 @@ class TestAsyncIPFS:
         assert imported['Root']['Cid']['/'] is not None
         assert reply['Cid']['/'] == imported['Root']['Cid']['/']
 
+        os.close(carfd)
         os.unlink(filecar)
 
     @pytest.mark.asyncio
+    @pytest.mark.skipif(sys.version_info < (3, 11),
+                        reason='Need python >= 3.11 for CAR decoding')
+    async def test_car_fs_export(self, event_loop, ipfsdaemon, iclient,
+                                 tmpdir, testfile1):
+        """
+        Test unpacking a UnixFS CAR export to a directory
+        by using the /dag/export endpoint
+        """
+
+        if await iclient.agent_version_get() < \
+                aioipfs.IpfsDaemonVersion('0.20.0'):
+            pytest.skip('Not testing CAR export for this version of kubo')
+
+        dst = Path(tmpdir).joinpath('unpacked')
+        dst.mkdir(parents=True, exist_ok=True)
+
+        cids = [added['Hash'] async for added in
+                iclient.add(str(testfile1), wrap=True, cid_version=1)]
+        top_cid = cids[-1]
+
+        path = Path(tmpdir).joinpath('export.car')
+        await iclient.dag.export(top_cid, output_path=path)
+
+        assert path.is_file()
+
+        # Test the function that reads a CAR file
+        stream = util.car_open(path)
+        assert stream
+
+        data = await util.car_bytes(stream, top_cid)
+        assert data == b'POIEKJDOOOPIDMWOPIMPOWE()=ds129084bjcy'
+
+        assert await iclient.dag.export_to_directory(top_cid, dst) is True
+
+        fp = dst.joinpath(cids[0])
+        assert fp.is_file()
+        assert fp.read_text() == 'POIEKJDOOOPIDMWOPIMPOWE()=ds129084bjcy'
+
+    @pytest.mark.asyncio
+    @pytest.mark.skipif(platform.system() == 'Windows',
+                        reason='This kubo API is not available on your OS')
     async def test_diag(self, event_loop, ipfsdaemon, iclient, tmpdir):
         reply = await iclient.diag.sys()
         assert 'diskinfo' in reply
-        await iclient.close()
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize('data', [b'0123456789'])
@@ -465,7 +460,19 @@ class TestAsyncIPFS:
         assert raw.decode() == '456789'
         raw = await iclient.cat(entry['Hash'], offset=2, length=3)
         assert raw.decode() == '234'
-        await iclient.close()
+
+    @pytest.mark.asyncio
+    async def test_get(self, event_loop, ipfsdaemon,
+                       iclient, testfile2, tmpdir):
+        cid: str = None
+
+        async for reply in iclient.add(str(testfile2)):
+            cid = reply['Hash']
+
+        result = await iclient.get(cid, dstdir=tmpdir)
+
+        assert result is True
+        assert cid in os.listdir(tmpdir)
 
     @pytest.mark.asyncio
     async def test_multiget(self, event_loop, ipfsdaemon,
@@ -487,8 +494,6 @@ class TestAsyncIPFS:
                 status, read, clength = result
                 assert status in [0, 1]
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     async def test_multibase(self, event_loop, ipfsdaemon, iclient,
                              tmpdir, testfile1):
@@ -516,8 +521,6 @@ class TestAsyncIPFS:
         reply = await iclient.multibase.transcode(str(encp))
         assert isinstance(reply, str)
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     @pytest.mark.parametrize('topic', ['aioipfs.pytest'])
     @pytest.mark.parametrize('msgdata', ['test',
@@ -583,8 +586,6 @@ class TestAsyncIPFS:
         await asyncio.sleep(0.5)
         assert t.result() is True
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     async def test_routing(self, event_loop, ipfsdaemon, iclient):
         if await iclient.agent_version_get() < \
@@ -599,14 +600,11 @@ class TestAsyncIPFS:
         provs = [p async for p in iclient.routing.findprovs(reply['Hash'])]
         assert len(provs) > 0
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     async def test_stats(self, event_loop, ipfsdaemon, iclient):
         await iclient.stats.bw()
         await iclient.stats.bitswap()
         await iclient.stats.repo()
-        await iclient.close()
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize('protocol', ['/x/test'])
@@ -630,7 +628,6 @@ class TestAsyncIPFS:
         await iclient.p2p.listener_close(protocol)
         listeners = await iclient.p2p.listener_ls()
         assert listeners['Listeners'] is None
-        await iclient.close()
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize('protocol', ['/x/test'])
@@ -644,8 +641,6 @@ class TestAsyncIPFS:
                                             allow_loopback=True) as ctx:
             assert ctx.maddr == Multiaddr(address)
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     @pytest.mark.parametrize('keysize', [2048, 4096])
     async def test_keys(self, event_loop, ipfsdaemon, iclient,
@@ -671,20 +666,16 @@ class TestAsyncIPFS:
         )
         assert reply['Name'] == impname
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     async def test_bitswap(self, event_loop, ipfsdaemon, iclient):
         await iclient.bitswap.wantlist()
         stats = await iclient.bitswap.stat()
         assert 'Wantlist' in stats
         assert 'DataSent' in stats
-        await iclient.close()
 
     @pytest.mark.asyncio
     async def test_filestore(self, event_loop, ipfsdaemon, iclient):
         await iclient.filestore.dups()
-        await iclient.close()
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize('obj', [b'0123456789'])
@@ -709,7 +700,6 @@ class TestAsyncIPFS:
                                   offset=5)
         data = await iclient.files.read('/test3', offset=5, count=3)
         assert data == otro
-        await iclient.close()
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize('obj', [b'0123456789'])
@@ -731,6 +721,13 @@ class TestAsyncIPFS:
     async def test_object(self, event_loop, ipfsdaemon, iclient, obj1, obj2,
                           testfile2):
         """ Unsure if this is correct """
+
+        if await iclient.agent_version_get() >= \
+                aioipfs.IpfsDaemonVersion('0.28.0'):
+            # Many of the 'object' RPC API methods are being deprecated
+            # starting with kubo v0.28.0
+            pytest.skip('This API is deprecated for this kubo version')
+
         obj1Ent = await iclient.add_bytes(obj1)
         obj2Ent = await iclient.add_bytes(obj2)
         obj = await iclient.object.new()
@@ -758,7 +755,37 @@ class TestAsyncIPFS:
         rm = await iclient.object.patch.rm_link(r2['Hash'], 'obj1')
         dag = await iclient.object.get(rm['Hash'])
         assert len(dag['Links']) == 1
-        await iclient.close()
+
+    @pytest.mark.asyncio
+    async def test_name_inspect(self, event_loop, ipfsdaemon, iclient):
+        """
+        Run name inspect on the node's IPNS key
+        """
+
+        if await iclient.agent_version_get() < \
+                aioipfs.IpfsDaemonVersion('0.20.0'):
+            # /api/v0/name/inspect was introduced in kubo v0.19.0
+            pytest.skip('RPC endpoint not available')
+
+        nid = (await iclient.id())['ID']
+        record = await iclient.routing.get(f'/ipns/{nid}')
+
+        with open('ipnsr.bin', 'w+b') as ipnsr:
+            ipnsr.write(base64.b64decode(record['Extra']))
+
+        result = await iclient.name.inspect('ipnsr.bin')
+        assert result['Entry']['Value']
+        assert result['Entry']['Validity']
+
+        # Try by passing a Path
+        result = await iclient.name.inspect(Path('ipnsr.bin'))
+        assert 'Entry' in result
+        assert result['Entry']['Value']
+        assert result['Entry']['Validity']
+
+        # Pass an invalid value type
+        with pytest.raises(ValueError):
+            await iclient.name.inspect(42)
 
     @pytest.mark.asyncio
     async def test_config(self, event_loop, ipfsdaemon, iclient, tmpdir):
@@ -787,8 +814,6 @@ class TestAsyncIPFS:
         result = await iclient.config.config('Bootstrap')
         assert result['Value'] == []
 
-        await iclient.close()
-
     @pytest.mark.asyncio
     async def test_cidapi(self, event_loop, ipfsdaemon, iclient, testfile1):
         async for added in iclient.add(str(testfile1), cid_version=1):
@@ -802,7 +827,34 @@ class TestAsyncIPFS:
         await iclient.cid.codecs()
         await iclient.cid.bases()
         await iclient.cid.hashes()
-        await iclient.close()
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize('pin_name', ['pintest'])
+    async def test_pin(self, event_loop, ipfsdaemon, iclient, pin_name):
+        entry = await iclient.add_bytes(b'Test', pin=False)
+
+        if await iclient.agent_version_get() >= \
+                aioipfs.IpfsDaemonVersion('0.26.0'):
+            """
+            kubo >= 0.26.0 supports optional pin names
+
+            Pin the object with a pin name and check that the entry
+            has the correct name when listing the pins
+            """
+
+            resp = [e async for e in iclient.pin.add(
+                entry['Hash'],
+                name=pin_name
+            )]
+            assert len(resp) > 0
+
+            pins = await iclient.pin.ls(names=True)
+            pine = pins['Keys'].get(entry['Hash'])
+
+            assert pine['Name'] == pin_name
+        else:
+            resp = [e async for e in iclient.pin.add(entry['Hash'])]
+            assert len(resp) > 0
 
     @pytest.mark.asyncio
     @pytest.mark.skip(reason='This test relies on specific network conditions')
@@ -843,4 +895,23 @@ class TestAsyncIPFS:
         res = await iclient.pin.remote.service.ls()
         assert len(res['RemoteServices']) == 0
 
-        await iclient.close()
+
+class TestMultipart:
+    def test_dirlisting(self, dir_hierarchy2):
+        def find(name: str, data):
+            for entry in data:
+                _name, _fd, _ctype = entry[1]
+                if _name == f'{dir_hierarchy2.name}/{name}':
+                    return entry
+
+        names = DirectoryListing(str(dir_hierarchy2), hidden=True).genNames()
+
+        assert find('README.txt', names)
+        assert find('README2.txt', names)
+        assert find('d/.e/f/.file3', names)
+        assert find('a/b/.c', names)
+        assert find('.file2', names)
+
+        names = DirectoryListing(str(dir_hierarchy2), hidden=False).genNames()
+
+        assert find('.file2', names) is None
